------------------------------------------------------------
-- StereoProcess: Build Stereo Flow
--
-- flow script
--
-- This script will build a stereo flow using LB_Tools StereoSwitch
-- 
--
--
-- KNOWN ISSUES
-- 
--
--
-- written by Laurens Bekaert (laurens.bekaert@aliendesigns.be)
-- written  : Januari 26th, 2016
-- updated : 02/11/2016 , 02/12/2016 , 30/01/2017
-- changes : Changed for loop to use pairs() and not ask for table value directly -- Added correct color for left and right loader to match anaglyph
------------------------------------------------------------


-- if the fusion variable isn't available we aren't running inside fusion
if not fusion then
	print("This is a Flow script, it should be run from within Digital Fusion")
end


function confExt(filepath,srchFor,srchTo)
	--If we would want to cast to lower to minimize search patterns
	--filepath = string.lower(filepath)
	--srchFor = string.lower(srchFor)
	--srchTo = string.lower(srchTo)
	findStart, findEnd	= string.find( filepath, srchFor, 1, true )
	print(findStart, findEnd, filepath)
	if findStart == nil then return filepath end
	
	-- build the new filename using strStart
	newclip =	string.sub(filepath, 1, findStart - 1) .. 
				srchTo ..
				string.sub(filepath, findEnd + 1)
				
	return newclip
end

eyes = {"Left", "Right"}

ret = comp:AskUser("Stereo Process Flow", {	
	{"Build",  "Dropdown", Options = eyes},
	--{"Savers", Name = "Change saver output", "Checkbox", Default = 1},
	{"instructions", Name="Instructions", "Text", Default="This script will build a stereo flow by adding loaders and LB_Tools StereoSwitch nodes using selected input from the dropdown menu. \n\nIt will search for stero naming patterns and change loader clip names to their respective counterparts. \n\nWhen 'mono' is found in the comments of the tool it will be skipped.", Wrap=true, Lines=10, ReadOnly=true},
	{"Stereo patterns", Name="Patterns", "Text", Default="Right\nright\n_R_\n_r_\nHe folder pattern R_\nDp pattern _r.\n\nLeft\nleft\n_L_\n_l_\nHe folder pattern L_\nDp pattern _l.", Wrap=true, Lines=14, ReadOnly=true}
	})

-- did they cancel?
if ret == nil then
	return
end	
	
if ret then
	print()
	print("Building Stereo Flow. Adding ".. eyes[ret.Build + 1] .." Eye Loaders and StereoSwitcher")
	print("-----------------------------------------------------------------------------------------")
	print()

ldrs = composition:GetToolList(false, "Loader")

composition:StartUndo("Build Stereo Script")
composition:Lock()
if table.getn( ldrs ) > 0 then
	for i, tool in pairs(ldrs) do --changed 02/11/2016 added pairs()
		print("Found a Loader")
		if string.find(tool.Comments[1], "mono") then		
			print("It's marked mono, skip loader\n")

		else
			
			if eyes[ret.Build + 1] == "Left" then
				
				local posX, posY = composition.CurrentFrame.FlowView:GetPos(tool)
				
				local new_loader = composition:AddTool("Loader", posX + 2 , posY - 1)
				new_loader:SetAttrs({TOOLS_Name = tool:GetAttrs()['TOOLS_Name'] .. "_Left", TOOLB_NameSet = true})
				new_loader.TileColor = {R = 255/256, G = 128/256, B = 128/256} -- RED, adding left loader --Added correct color for left and right loader to match anaglyph
				print("Created Left Loader for " .. tool:GetAttrs()['TOOLS_Name'])
				
				tool.TileColor = {R = 64/256, G = 128/256, B = 128/256} -- Current loader is right cyaan -- Added correct color for left and right loader to match anaglyph
				
				clipTable = tool:GetAttrs().TOOLST_Clip_Name --30/01/2017
				startTime = tool:GetAttrs().TOOLNT_Clip_Start --30/01/2017
				--newClip = tool.Clip[TIME_UNDEFINED] --changed 12/09/2016
				for i = 1, table.getn( clipTable ) do --30/01/2017
					clip = clipTable[i] --30/01/2017
					print(clip) 
					--newClip = clip
				
					print("Setting new loader path to Left")
					--print(newClip)
							newclip = confExt( clip , "Right" , "Left" )
							newclip = confExt( newclip , "right" , "left" )
							newclip = confExt( newclip , "_R_" , "_L_" )
							newclip = confExt( newclip , "_r_" , "_l_" )
							--Special Check For HARVIE path
							newclip = confExt( newclip , "R_" , "L_" )
							newclip = confExt( newclip , "_r." , "_l." )

							if newclip ~= nil then
								if newclip == clip then
								print( "No Stereo Pattern Found\n\nEMPTY LOADER\n" ) 
								else
								new_loader.Clip[startTime[i]] = newclip --30/01/2017
								print("Path changed to Left")
								end
							else
								print( "New clip output NILL\n\n" )
							end
				end
						
				
				--	new_loader.Clip[TIME_UNDEFINED] = newClip
				--	print("Path changed to Left")
				--end
				--if firstframe ~= nil and globalin ~= firstframe then
					--print("First frame at ".. globalin)
				--end
				
				local new_switcher = composition:AddTool("Aliendesigns.StereoSwitch", posX + 2 , posY )
				new_switcher:SetAttrs({TOOLS_Name = "StereoSwitch_" ..tool:GetAttrs()['TOOLS_Name'] , TOOLB_NameSet = true})
				new_switcher.LeftImage:ConnectTo(new_loader.Output)
				
				--find left loader output table and save to substitute w switcher output
				
				connectedInputs = tool.Output:GetConnectedInputs()
				if table.getn(connectedInputs) > 0 then
					print("found outputs")
					for j, inputs in pairs(connectedInputs) do --changed 02/11/2016 added pairs()
                    inputs:ConnectTo(new_switcher.Output)
					end
					print("connected outputs \n")
				else
					print("NO INPUTS FOUND LINKED TO ORIGINAL LOADER \n")
                end
				
				--connect existing loader output to switcher
				new_switcher.RightImage:ConnectTo(tool.Output)
				--Building left image means that we have a working right so make sure switcher is set to right
				new_switcher.Input = "Right"
				--composition:Unlock()
			else

				local posX, posY = composition.CurrentFrame.FlowView:GetPos(tool)
				
				local new_loader = composition:AddTool("Loader", posX + 2 , posY + 2)
				new_loader:SetAttrs({TOOLS_Name = tool:GetAttrs()['TOOLS_Name'] .. "_Right", TOOLB_NameSet = true})
				new_loader.TileColor = {R = 64/256, G = 128/256, B = 128/256} -- CYAAN, adding right loader --Added correct color for left and right loader to match anaglyph
				print("Created Right Loader for " .. tool:GetAttrs()['TOOLS_Name'])
				
				tool.TileColor = {R = 255/256, G = 128/256, B = 128/256} -- Current loader is left red -- Added correct color for left and right loader to match anaglyph
				
				clipTable = tool:GetAttrs().TOOLST_Clip_Name --30/01/2017
				startTime = tool:GetAttrs().TOOLNT_Clip_Start --30/01/2017
				
				for i = 1, table.getn( clipTable ) do --30/01/2017
					clip = clipTable[i] --30/01/2017
					print(clip) 
				
					print("Setting new loader path to Right")
							newclip = confExt( clip , "Left" , "Right" )
							newclip = confExt( newclip , "left" , "right" )
							newclip = confExt( newclip , "_L_" , "_R_" )
							newclip = confExt( newclip , "_l_" , "_r_" )
							--Special Check For HARVIE path
							newclip = confExt( newclip , "L_" , "R_" )
							newclip = confExt( newclip , "_l." , "_r." )

						if newclip ~= nil then
							if newclip == clip then
							print( "No Stereo Pattern Found\n\nEMPTY LOADER\n" ) 
							else
							new_loader.Clip[startTime[i]] = newclip --30/01/2017
							print("Path changed to Right")
							end
						else
							print( "ERROR: New clip output NILL\n\n" )
						end
				end
						
				local new_switcher = composition:AddTool("Aliendesigns.StereoSwitch", posX + 2 , posY )
				new_switcher:SetAttrs({TOOLS_Name = "StereoSwitch_" ..tool:GetAttrs()['TOOLS_Name'] , TOOLB_NameSet = true})
				new_switcher.RightImage:ConnectTo(new_loader.Output)
				
				--find right loader output table and save to substitute w switcher output
				connectedInputs = tool.Output:GetConnectedInputs()
				if table.getn(connectedInputs) > 0 then
					print("found outputs")
					for j, inputs in pairs(connectedInputs) do --changed 02/11/2016 added pairs()
                    inputs:ConnectTo(new_switcher.Output)
					end
					print("connected outputs \n")
				else
					print("NO INPUTS FOUND LINKED TO ORIGINAL LOADER \n")
                end
				
				--connect existing loader output to switcher
				
				new_switcher.LeftImage:ConnectTo(tool.Output)
				--composition:Unlock()
			end
		end
		
	end
else
	print("No Loaders Found")
end
print("\nScript had ended\n")
end
composition:Unlock()
composition:EndUndo(true)