------------------------------------------------------------
-- StereoProcess: Switch Flow To Other Eye
--
-- flow script
--
-- This script will adjust all LB_Tools StereoSwitch nodes in a flow to the selected input from the dropdown menu
-- When Change Saver Output checkbox is selected it will search for stero naming patterns and change them to their respective counterparts
--
--
-- KNOWN ISSUES
-- If any of the search patterns are in the name of the file but not related to stereo workflow they will get changed anyways and the saver output might not be correct
-- If a search pattern is found twice only the first instance will be changed
-- fe. ..renders2D\right\render_right_000.jpg will become ..renders2D\left\render_right_000.jpg (you can alwas run the process a second time)
--
--
-- written by Laurens Bekaert (laurens.bekaert@aliendesigns.be)
-- written  : Januari 26th, 2016
-- updated : 02/11/2016 , 30/01/2017
-- changes : Changed for loop to use pairs() and not ask for table value directly
------------------------------------------------------------


-- if the fusion variable isn't available we aren't running inside fusion
if not fusion then
	print("This is a Flow script, it should be run from within Digital Fusion")
end


function confExt(filepath,srchFor,srchTo)
	--If we would want to cast to lower to minimize search patterns
	--filepath = string.lower(filepath)
	--srchFor = string.lower(srchFor)
	--srchTo = string.lower(srchTo)
	findStart, findEnd	= string.find( filepath, srchFor, 1, true )
	print(findStart, findEnd, filepath)
	if findStart == nil then return filepath end
	
	-- build the new filename using strStart
	newclip =	string.sub(filepath, 1, findStart - 1) .. 
				srchTo ..
				string.sub(filepath, findEnd + 1)
				
	return newclip
end


eyes = {"Left", "Right"}

ret = comp:AskUser("Stereo Process Flow", {	
	{"Switch",  "Dropdown", Options = eyes},
	{"Savers", Name = "Change saver output", "Checkbox", Default = 1},
	{"instructions", Name="Instructions", "Text", Default="This script will adjust all LB_Tools StereoSwitch nodes in a flow to the selected input from the dropdown menu. \n\nWhen Change Saver Output checkbox is selected it will search for stero naming patterns and change them to their respective counterparts. \n\nWhen 'mono' is found in the comments of the tool it will be skipped.", Wrap=true, Lines=10, ReadOnly=true},
	{"Stereo pattens", Name="Patterns", "Text", Default="Right\nright\n_R_\n_r_\n\nLeft\nleft\n_L_\n_l_", Wrap=true, Lines=10, ReadOnly=true}
	})

-- did they cancel?
if ret == nil then
	return
end
	
if ret then
	print()
	print("Changing Stereo Switchers To ".. eyes[ret.Switch + 1])
	print("---------------------------------------------------")
	print()

	-- ((ret.Savers ==1)) will return true if the 
	-- selected checkbox is enabled.....

swtch = composition:GetToolList(false, "Aliendesigns.StereoSwitch")
composition:StartUndo("Switch Stereo Script")
composition:Lock()


if table.getn( swtch ) > 0 then
	for i, tool in pairs(swtch) do  --changed 02/11/2016 added pairs()
		print("Found a StereoSwitch")
		if string.find(tool.Comments[1], "mono") then --30/01/2017
			print("It's marked mono, skip switch\n")
		else
			if eyes[ret.Switch + 1] == "Left" then
				print("Switch is set to Left\n")
				tool.Input = "Left"
			else
				tool.Input = "Right"
				print("Switch is set to Right\n")
			end
		end
	end
else
	print("No StereoSwitch found\n")
end
end

if (ret.Savers ==1) then
	print()
	print("Changing Savers ")
	print("---------------------------------------------------")
	print()
	
svs = composition:GetToolList(false, "Saver")

if table.getn( svs ) > 0 then
	for i, tool in pairs(svs) do  --changed 02/11/2016 added pairs()
		print("Found a Saver")
		if string.find(tool.Comments[1], "mono") then		
			print("It's marked mono, skip saver\n")
		else
			clipTable = tool:GetAttrs().TOOLST_Clip_Name --30/01/2017
			startTime = tool:GetAttrs().TOOLNT_Clip_Start --30/01/2017
			for j = 1, table.getn( clipTable ) do --30/01/2017
					clip = clipTable[j] --30/01/2017
					print(clip) 
				if eyes[ret.Switch + 1] == "Left" then
					print("Setting path to Left")
					newclip = confExt( clip , "Right" , "Left" ) --30/01/2017
					newclip = confExt( newclip , "right" , "left" )
					newclip = confExt( newclip , "_R_" , "_L_" )
					newclip = confExt( newclip , "_r_" , "_l_" )

					if newclip ~= nil then
						tool.Clip[startTime[j]] = newclip --30/01/2017
					end
					print("Path changed to Left")
					--tool.Clip = "Comp:..\\renders2d\\left\\Master_L_000000.exr"
				else
					print("Setting path to Right")
					newclip = confExt( clip , "Left" , "Right" ) --30/01/2017
					newclip = confExt( newclip , "left" , "right" )
					newclip = confExt( newclip , "_L_" , "_R_" )
					newclip = confExt( newclip , "_l_" , "_r_" )

					if newclip ~= nil then
						tool.Clip[startTime[j]] = newclip --30/01/2017
					end
					print("Path changed to Right")
					--tool.Clip = "Comp:..\\renders2d\\right\\Master_R_000000.exr"
				end
			end
		end
	end
else
	print("No Savers found\n")
end
print("\nScript had ended\n")
end
composition:Unlock()
composition:EndUndo(true)